@@ -31,6 +31,7 @@ class ConnectionClass():
   reconnectFadeoffFactor = None
 
   _connectIfNeededLock = None
+  _onDisconnectedInProgress = None
 
   def __init__(self, fullConnectionDetails, recieveFunction, reconnectMaxRetries, reconectInitialSecondsBetweenTries, reconnectFadeoffFactor, description="Initial"):
     self._connectIfNeededLock = threading.Lock()
@@ -44,12 +45,18 @@ class ConnectionClass():
     self.reconectInitialSecondsBetweenTries = reconectInitialSecondsBetweenTries
     self.reconnectFadeoffFactor = reconnectFadeoffFactor
     self.stompConnection = None
+    self._onDisconnectedInProgress = False
 
     self._connectIfNeeded(description=description)
 
+  def log(self, string):
+    print("STOMP connection: " + string)
+
   def _connectIfNeeded(self, description):
+    self.log("connectIfNeeded start (" + description + ")")
     self._connectIfNeededLock.acquire(blocking=True, timeout=-1)
     if self.connected:
+      self.log("connectIfNeeded already connected(" + description + ")")
       self._connectIfNeededLock.release()
       return
     self.stompConnection = stomp.Connection(
@@ -100,25 +107,40 @@ class ConnectionClass():
     raise Exception(message)
 
   def _onDisconnected(self):
+    if self._onDisconnectedInProgress:
+      return
+    self._onDisconnectedInProgress = True
     try:
+      self.log("onDisconnected")
       if self.closed:
+        self.log("onDisconnected - closing so ignored")
+        self._onDisconnectedInProgress = False
         return
       self._connectIfNeededLock.acquire(blocking=True, timeout=-1)
       self.connected = False
+      self.stompConnection.disconnect()
       self.stompConnection = None
       self._connectIfNeededLock.release()
       # If we have subscribers then we must reconnect and register all the subscriptions again
       #  otherwise we cna wait and only reconnect when sendStringMessage or registerSubscription is called
       if len(self.registeredSubscriptions) == 0:
+        self.log("onDisconnected - no subscriptions so not reconnecting at this point")
+        self._onDisconnectedInProgress = False
         return
+
+      self.log("onDisconnected - There are subscriptions so connecgin if needed")
       self.retryWrapperAround_connectIfNeeded(description="_onDisconnected")
 
       #now we have connected re-register all subscriptions on new connection
+      self.log("onDisconnected - Resubscribing")
       for internalDestination in self.registeredSubscriptions:
         self.registeredSubscriptions[internalDestination].subscribeToStompConnection(self.stompConnection)
+        self._onDisconnectedInProgress = False
     except Exception as excepti:
       self.thrownException = MqClientThreadHealthCheckExceptionClass("Exception thrown in stomp")
       raise excepti
+    finally:
+      self._onDisconnectedInProgress = False
 
   def _onMessage(self, headers, message):
     registeredSubscription = None
@@ -149,6 +171,7 @@ class ConnectionClass():
       print("Got stomp.exception.NotConnectedException but on disconnect was never called - trying one more time")
       self._connectIfNeededLock.acquire(blocking=True, timeout=-1)
       self.connected = False
+      self.stompConnection.disconnect()
       self.stompConnection = None
       self._connectIfNeededLock.release()
       self._connectIfNeeded(description="sendStringMessageRETRY")
@@ -174,6 +197,7 @@ class ConnectionClass():
   def close(self, wait):
     self.closed = True
     self.stompConnection.disconnect()
+    self.stompConnection = None
 
   def healthCheck(self):
     if self.thrownException is not None:
